package edu.travel.utils.tree;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 树形菜单构建工具
 *
 */
public class MyTreeUtil {
    /**
     * 构建树形工具
     *
     * @param resource 数据源
     * @param <T>      node
     * @return 树形
     */
    public static <T extends TreeNode> List<T> buildTree(List<T> resource) {
        Map<String, List<TreeNode>> groups = ifNullDefault(resource)
                .stream()
                .collect(Collectors.groupingBy(TreeNode::getParentId, LinkedHashMap::new, Collectors.toList()));
        return resource.stream()
                .filter(Objects::nonNull)
                .peek(node -> node.setChildren(groups.get(node.getId())))
                .filter(TreeNode::getRootNode)
                .collect(Collectors.toList());
    }

    /**
     * 防止空指针异常处理
     *
     * @param list 集合数据
     * @return 如果为空返
     */
    private static <L> List<L> ifNullDefault(List<L> list) {
        return Optional.ofNullable(list).orElseGet(Collections::emptyList);
    }
}
