package edu.travel.commodity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.adapter.service.tenant.TenantAdapter;
import edu.travel.commodity.entity.ShopReview;
import edu.travel.commodity.mapper.ShopReviewMapper;
import edu.travel.commodity.service.ShopReviewService;
import edu.travel.commodity.utils.PageUtil;
import edu.travel.commodity.utils.StringToListUtil;
import edu.travel.dto.ProductSpecDto;
import edu.travel.exception.BaseException;
import edu.travel.remote.feign.mode.vo.tenant.EduTenantVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.vo.ShopReviewVo;
import edu.travel.vo.SpecOneVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@Service
public class ShopReviewServiceImpl extends ServiceImpl<ShopReviewMapper, ShopReview> implements ShopReviewService {
    @Autowired
    private TenantAdapter tenantAdapter;
    @Override
    public Page<ShopReviewVo> getShopReview(ProductSpecDto params) {
        LambdaQueryWrapper<ShopReview> query = Wrappers.<ShopReview>lambdaQuery()
                .eq(ShopReview::getProductId, params.getProductId())
                .orderByDesc(ShopReview::getIsTop).orderByDesc(ShopReview::getCreateTime);

        Page<ShopReview> page = page(new Page<>(params.getCurrentPage(), params.getPageSize()), query);

        Page<ShopReviewVo> pageEntity = PageUtil.toPageEntity(page, ShopReviewVo.class);
        List<ShopReviewVo> records = pageEntity.getRecords();
        if(ObjectUtil.isEmpty(records)){
            return pageEntity;
        }
        Set<String> collect = records.stream().map(ShopReviewVo::getUserId).collect(Collectors.toSet());
        RPCBaseResponse<List<EduTenantVo>> tenantByIds = tenantAdapter.getTenantByIds(collect);
        //继续抛出
        if(tenantByIds.getCode()!=200){
            throw new BaseException(tenantByIds.getCode(), tenantByIds.getMsg());
        }
        List<EduTenantVo> data = tenantByIds.getData();
        HashMap<String, EduTenantVo> users = new HashMap<>();
        for (EduTenantVo datum : data) {
            users.put(datum.getId(), datum);
        }
        for (ShopReviewVo record : records) {
            List<String> list = StringToListUtil.toList(record.getImage());
            record.setImageUrls(list);
            SpecOneVo bean = JSONUtil.toBean(record.getSkuDetails(), SpecOneVo.class);
            record.setSpecOneVo(bean);
            EduTenantVo eduTenantVo = users.get(record.getUserId());
            record.setEduTenantVo(eduTenantVo);
        }
        return pageEntity;
    }
}
